"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const cross_fetch_1 = require("cross-fetch");
const fs = require("fs");
const git = require("isomorphic-git");
const path = require("path");
const punicaException_1 = require("../exception/punicaException");
const fileSystem_1 = require("../utils/fileSystem");
// tslint:disable:no-console
class Box {
    async init(initToPath) {
        const repoUrl = 'https://github.com/punica-box/punica-init-default-box.git';
        await this.gitClone(repoUrl, initToPath);
        this.handleIgnorance(initToPath);
        console.log('Unbox successful. Enjoy it!');
    }
    async handleIgnorance(repoToPath = '') {
        console.log('Unpacking...');
        const boxIgnoreFilePath = path.join(repoToPath, 'punica-box.json');
        let boxIgnoreFiles;
        if (!fs.existsSync(boxIgnoreFilePath)) {
            return;
        }
        try {
            const f = fs.readFileSync(boxIgnoreFilePath);
            const json = JSON.parse(f.toString());
            boxIgnoreFiles = json.ignore;
            fileSystem_1.removeFileIfExists(boxIgnoreFilePath);
            if (boxIgnoreFiles !== undefined) {
                for (const file of boxIgnoreFiles) {
                    const filePath = path.join(repoToPath, file);
                    try {
                        fileSystem_1.ensureRemoveDirIfExists(filePath);
                        fileSystem_1.removeFileIfExists(filePath);
                    }
                    catch (_2) {
                        console.warn(`Cant delete ${filePath}.`);
                    }
                }
            }
        }
        catch (_) {
            console.warn(`Cant process ${repoToPath}.`);
            throw _;
        }
    }
    async gitClone(repoUrl, repoToPath = '') {
        console.log('Downloading...');
        if (fs.existsSync(repoToPath)) {
            if (fs.statSync(repoToPath).isDirectory()) {
                if (fs.readdirSync(repoToPath).length > 0) {
                    throw punicaException_1.fileExistError();
                }
            }
            else {
                throw punicaException_1.fileExistError();
            }
        }
        try {
            await git.clone({ url: repoUrl, dir: repoToPath });
        }
        catch (e) {
            throw punicaException_1.networkError();
        }
    }
    generateRepoUrl(boxName) {
        const re1 = /^([a-zA-Z0-9-])+$/;
        const re2 = /^([a-zA-Z0-9-])+\/([a-zA-Z0-9-])+$/;
        if (re1.test(boxName)) {
            return `https://github.com/punica-box/${boxName}-box.git`;
        }
        else if (!re2.test(boxName)) {
            return `https://github.com/${boxName}.git`;
        }
        else {
            throw punicaException_1.invalidBoxName();
        }
    }
    async list() {
        const response = await cross_fetch_1.default('https://api.github.com/orgs/punica-box/repos');
        const json = await response.json();
        return json
            .map((repo) => repo.name)
            .filter((name) => name !== undefined)
            .filter((name) => name.endsWith('-box'))
            .map((name) => name.substr(0, name.length - '-box'.length));
    }
    async unbox(boxName, repoToPath = '') {
        const repoUrl = this.generateRepoUrl(boxName);
        try {
            await this.gitClone(repoUrl, repoToPath);
        }
        catch (e) {
            if (e instanceof punicaException_1.PunicaException) {
                if (e.code === 59000) {
                    console.log('Please check out your box name.');
                    return;
                }
            }
            throw e;
        }
        try {
            await this.handleIgnorance(repoToPath);
        }
        catch (e) {
            console.log('Clean work abort...');
            return;
        }
        console.log('Unbox successful. Enjoy it!');
    }
}
exports.Box = Box;
//# sourceMappingURL=box.js.map