"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ontology_ts_crypto_1 = require("ontology-ts-crypto");
const util_1 = require("util");
function getParamName(param) {
    return param.name;
}
function getParamValue(param) {
    if (param === undefined) {
        return undefined;
    }
    return param.value;
}
function convertParams(invokeParams, abiInfo) {
    return abiInfo.parameters.map((abiParameter) => {
        const invokeParameter = getParamValue(invokeParams.find((ip) => getParamName(ip) === abiParameter.name));
        if (invokeParameter === undefined) {
            throw new Error('Missing parameter value.');
        }
        return convertParam(invokeParameter);
    });
}
exports.convertParams = convertParams;
function convertParam(param) {
    if (typeof param === 'boolean') {
        return param;
    }
    else if (typeof param === 'number') {
        return param;
    }
    else if (typeof param === 'string') {
        if (param.startsWith('ByteArray:')) {
            return new Buffer(param.substr('ByteArray:'.length), 'hex');
        }
        else if (param.startsWith('String:')) {
            return param.substr('String:'.length);
        }
        else if (param.startsWith('Address:')) {
            const address = ontology_ts_crypto_1.Address.fromBase58(param.substr('Address:'.length));
            return address.toArray();
        }
        else {
            // string parameters are more likely hex encoded
            return new Buffer(param, 'hex');
        }
    }
    else if (util_1.isArray(param)) {
        return param.map((child) => convertParam(child));
    }
    else if (typeof param === 'object') {
        // this is last, because other classes are also objects
        const entries = Object.entries(param);
        const converted = entries.map(([key, value]) => [key, convertParam(value)]);
        return new Map(converted);
    }
    else {
        throw new Error('Unsupported param type');
    }
}
exports.convertParam = convertParam;
function convertParamsStr(invokeParams, abiInfo) {
    return abiInfo.parameters.map((abiParameter) => {
        const invokeParameter = getParamValue(invokeParams.find((ip) => getParamName(ip) === abiParameter.name));
        if (invokeParameter === undefined) {
            throw new Error('Missing parameter value.');
        }
        return convertParamStr(invokeParameter);
    });
}
exports.convertParamsStr = convertParamsStr;
function convertParamStr(param) {
    if (typeof param === 'boolean') {
        return `${param}`;
    }
    else if (typeof param === 'number') {
        return `${param}`;
    }
    else if (typeof param === 'string') {
        if (param.startsWith('ByteArray:')) {
            return `new Buffer('${param.substr('ByteArray:'.length)}', 'hex')`;
        }
        else if (param.startsWith('String:')) {
            return `'${param.substr('String:'.length)}'`;
        }
        else if (param.startsWith('Address:')) {
            const address = ontology_ts_crypto_1.Address.fromBase58(param.substr('Address:'.length));
            return `new Buffer('${address.toArray().toString('hex')}', 'hex')`;
        }
        else {
            // string parameters are more likely hex encoded
            return `new Buffer('${param}', 'hex')`;
        }
    }
    else if (util_1.isArray(param)) {
        return `[${param.map((child) => convertParamStr(child)).join(', ')}]`;
    }
    else {
        throw new Error('Unsupported param type');
    }
}
exports.convertParamStr = convertParamStr;
//# sourceMappingURL=params.js.map